#ifndef TestSAN_H_
#define TestSAN_H_

#include "Cpp/BaseClasses/EmptyGroup.h"
#include "Cpp/BaseClasses/GlobalVariables.h"
#include "Cpp/BaseClasses/PreselectGroup.h"
#include "Cpp/BaseClasses/PostselectGroup.h"
#include "Cpp/BaseClasses/state/StructStateVariable.h"
#include "Cpp/BaseClasses/state/ArrayStateVariable.h"
#include "Cpp/BaseClasses/SAN/SANModel.h" 
#include "Cpp/BaseClasses/SAN/Place.h"
#include "Cpp/BaseClasses/SAN/ExtendedPlace.h"
extern Int current_R;
extern Int current_H;
extern UserDistributions* TheDistribution;

void MemoryError();


/*********************************************************************
               TestSAN Submodel Definition                   
*********************************************************************/

class TestSAN:public SANModel{
public:

class proposableActivity:public Activity {
public:

  Place* is_proposer;
  short* is_proposer_Mobius_Mark;
  Place* start;
  short* start_Mobius_Mark;
  Place* propose_i;
  short* propose_i_Mobius_Mark;

  double* TheDistributionParameters;
  proposableActivity();
  double Rate(){return 0;}
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
}; // proposableActivityActivity

class commitActivity:public Activity {
public:

  Place* committing;
  short* committing_Mobius_Mark;

  double* TheDistributionParameters;
  commitActivity();
  double Rate(){return 0;}
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
 bool precommit_successIP();
}; // commitActivityActivity

class prevotingActivity:public Activity {
public:

  Place* prevote_1;
  short* prevote_1_Mobius_Mark;
  Place* prevote_2;
  short* prevote_2_Mobius_Mark;
  Place* prevote_3;
  short* prevote_3_Mobius_Mark;
  Place* receive_proposal;
  short* receive_proposal_Mobius_Mark;
  Place* prevote;
  short* prevote_Mobius_Mark;

  double* TheDistributionParameters;
  prevotingActivity();
  double Rate(){return 0;}
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
 bool prevote_successIP();
}; // prevotingActivityActivity

class proposeActivity:public Activity {
public:

  Place* propose_i;
  short* propose_i_Mobius_Mark;
  Place* wait;
  short* wait_Mobius_Mark;

  double* TheDistributionParameters;
  proposeActivity();
  ~proposeActivity();
  double Rate(){return 0;}
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
  double DeterministicParamValue();
}; // proposeActivityActivity

class timeoutActivity:public Activity {
public:

  Place* wait;
  short* wait_Mobius_Mark;
  Place* prevote;
  short* prevote_Mobius_Mark;

  double* TheDistributionParameters;
  timeoutActivity();
  ~timeoutActivity();
  double Rate(){return 0;}
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
  double DeterministicParamValue();
}; // timeoutActivityActivity

  //List of user-specified place names
  Place* start;
  Place* is_proposer;
  Place* propose_i;
  Place* wait;
  Place* prevote;
  Place* receive_proposal;
  Place* prevote_1;
  Place* prevote_2;
  Place* prevote_3;
  Place* committing;
  ExtendedPlace<int>* precommit_1;
  ExtendedPlace<int>* precommit_2;
  ExtendedPlace<int>* precommit_3;

  // Create instances of all actvities
  proposableActivity proposable;
  commitActivity commit;
  prevotingActivity prevoting;
  proposeActivity propose;
  timeoutActivity timeout;
  //Create instances of all groups 
  PreselectGroup ImmediateGroup;
  PostselectGroup proposableGroup;
  PostselectGroup commitGroup;
  PostselectGroup prevotingGroup;

  TestSAN();
  ~TestSAN();
  void CustomInitialization();

  void assignPlacesToActivitiesInst();
  void assignPlacesToActivitiesTimed();
}; // end TestSAN

#endif // TestSAN_H_
